# Running AMD servers on DeTEE

Registering an AMD server on the DeTEE network will allow you to collect rewards by running VMs for the DeTEE users.

To be able to run the server on the network, it must support SEV SNP, meaning it must be part of these 3 generations:
- AMD EPYC 7003 (Milan)
- AMD EPYC 8004 (Siena)
- AMD EPYC 9004 (Genoa or Bergamo)
- AMD EPYC 9005 (Turin)

More information about the capabilities of each generation can be found on the [AMD website](https://www.amd.com/en/products/processors/server/epyc/infinity-guard.html). Our focus is strictly focused on the SEV-SNP capability.

The server must have a public IP.

## The Kernel

TL/DR: You need a new kernel, so install Fedora or Archlinux on the server. We will write here when newer generations are supported.

## DeTEE Daemon

Install the detee-snp-daemon from here (TODO: add link).

You will need to configure the daemon by modifying `/etc/detee/daemon/config.yaml`. The default configuration does not offer public IPs, and offers by default 16 vCPUs, 20GB of Memory and 400GB of storage. Modify these values depending on how many resources you want to rent to the network.

Here is a sample configuration that includes public IPv4 and IPv6 IPs:
```json
brain_url: "http://164.92.249.180:31337"
max_cores_per_vm: 8
max_vcpu_reservation: 24
max_mem_reservation_mb: 25000
network_interfaces:
  - driver: "MACVTAP"
    device: "eno8303"
    ipv4_ranges:
      - first_ip: "173.234.136.154"
        last_ip: "173.234.136.155"
        netmask: "27"
        gateway: "173.234.136.158"
      - first_ip: "173.234.137.17"
        last_ip: "173.234.137.17"
        netmask: "27"
        gateway: "173.234.137.30"
    ipv6_ranges:
      - first_ip: "2a0d:3003:b666:a00c:0002:0000:0000:0011"
        last_ip: "2a0d:3003:b666:a00c:0002:0000:0000:fffc"
        netmask: "64"
        gateway: "2a0d:3003:b666:a00c::1"
volumes:
  - path: "/var/lib/detee/"
    max_reservation_gb: 500
public_port_range:
  start: 30000
  end: 50000
max_ports_per_vm: 5
price: 20000
```

## Configuration details

The only network driver currently supported is MACVTAP. If you have a subnet that does not work with MACVTAP, please contact us on our [Discord Server](https://discord.gg/TSG2DKnw), and we will improve the code to allow you to host your network using IPVTAP or a Bridge setup.

The private IPv4 connection and port forwarding capabilities are enabled through QEMU. VMs will not be able to communicate with each other using this setup.

Please feel free to experiment with the price. A price of 20000 is recommended by our team in the testnet.

## Registering node

You will need the detee-cli (TODO: add link) to register your node under your DeTEE account. The rewards are collected by the owner (meaning the DeTEE wallet generated by the CLI) and not by the node. Follow the steps below to register the node.

When booting, the node will print it's public key in the logs. To access logs, run:
```
journalctl -fu detee-snp-daemon.service
```
In the logs, there will be a line similar to this:
```
Jan 26 00:59:13 detee-amd-1 detee-snp-daemon[764125]: [2025-01-25T22:59:13Z INFO  detee_snp_daemon::global] Loaded the following public key: 7Xw3RxbP5pvfjZ8U6yA3HHVSS9YXjKH5Vkas3JRbQYd9
```

Grab the key and use the following command in the detee-cli to register the node:
```
detee-cli account register-node 7Xw3RxbP5pvfjZ8U6yA3HHVSS9YXjKH5Vkas3JRbQYd9
```

You should be set.

## Easy way to get free tokens

- tell your friends about DeTEE
- convince them to write about us on social media so that they get an airdrop
- ask them to use their airdrop to create a VM on your server

<script type="text/javascript" id="hcb" src="/js/comments.js"></script><div id="HCB_comment_box"></div><link rel="stylesheet" type="text/css" href="/css/comments.css"/>
